package com.ruoyi.frequency.mallshoppingcar.controller;

import java.util.List;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.frequency.mallshoppingcar.entity.MallShoppingCar;
import com.ruoyi.frequency.mallshoppingcar.service.MallShoppingCarService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 购物车Controller
 * 
 * @author liwenlong
 * @date 2024-01-25
 */
@RestController
@RequestMapping("/frequency/mallshoppingcar")
public class MallShoppingCarController extends BaseController
{
    @Autowired
    private MallShoppingCarService mallShoppingCarService;

    /**
     * 查询购物车列表
     */
    @PreAuthorize("@ss.hasPermi('frequency:mallshoppingcar:list')")
    @GetMapping("/list")
    public TableDataInfo<MallShoppingCar> list(MallShoppingCar mallShoppingCar)
    {
        startPage();
        List<MallShoppingCar> list = mallShoppingCarService.selectMallShoppingCarList(mallShoppingCar);
        return getDataTable(list);
    }

    /**
     * 导出购物车列表
     */
    @PreAuthorize("@ss.hasPermi('frequency:mallshoppingcar:export')")
    @Log(title = "购物车", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(MallShoppingCar mallShoppingCar)
    {
        List<MallShoppingCar> list = mallShoppingCarService.selectMallShoppingCarList(mallShoppingCar);
        ExcelUtil<MallShoppingCar> util = new ExcelUtil<MallShoppingCar>(MallShoppingCar.class);
        return util.exportExcel(list, "购物车数据");
    }

    /**
     * 获取购物车详细信息
     */
    @PreAuthorize("@ss.hasPermi('frequency:mallshoppingcar:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult<MallShoppingCar> getInfo(@PathVariable("id") Long id)
    {
        return AjaxResult.success(mallShoppingCarService.selectMallShoppingCarById(id));
    }

    /**
     * 新增购物车
     */
    @PreAuthorize("@ss.hasPermi('frequency:mallshoppingcar:add')")
    @Log(title = "购物车", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody MallShoppingCar mallShoppingCar)
    {
        return toAjax(mallShoppingCarService.insertMallShoppingCar(mallShoppingCar));
    }

    /**
     * 修改购物车
     */
    @PreAuthorize("@ss.hasPermi('frequency:mallshoppingcar:edit')")
    @Log(title = "购物车", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody MallShoppingCar mallShoppingCar)
    {
        return toAjax(mallShoppingCarService.updateMallShoppingCar(mallShoppingCar));
    }

    /**
     * 删除购物车
     */
    @PreAuthorize("@ss.hasPermi('frequency:mallshoppingcar:remove')")
    @Log(title = "购物车", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}/{delFlag}")
    public AjaxResult remove(@PathVariable("ids") Long[] ids,@PathVariable("delFlag") Integer delFlag)
    {
        return toAjax(mallShoppingCarService.deleteMallShoppingCarByIds(ids,delFlag));
    }
}
